@@ -73,7 +73,7 @@ static void press(Entry *e);
 static void run(void);
 static void setup(void);
 static int textnw(const char *text, uint len);
-static void unpress(void);
+static void unpress(Entry *e);
 static void updateentries(void);
 
 /* variables */
@@ -91,13 +91,18 @@ static void (*handler[LASTEvent]) (XEvent *) = {
 static Display *dpy;
 static DC dc;
 static Window root, win;
-static Bool running = True;
-static int ww = 0, wh = 0, wx = 0, wy = 0;
+static Bool running = True, horizontal = False;
+/*
+ * ww = window width; www = wanted window width; wh = window height;
+ * wx = window x position; wy = window y position;
+ */
+static int ww = 0, www = 0, wh = 0, wx = 0, wy = 0;
 static char *name = "thingmenu";
 
 Entry **entries = NULL;
 int nentries = 0;
 int oneshot = 1;
+Bool ispressing = 0;
 
 /* configuration, allows nested code to access above variables */
 #include "config.h"
@@ -114,11 +119,19 @@ motionnotify(XEvent *e)
 				&& ev->y > entries[i]->y
 				&& ev->y < entries[i]->y + entries[i]->h) {
 			if (entries[i]->highlighted != True) {
-				entries[i]->highlighted = True;
+				if (ispressing) {
+					entries[i]->pressed = True;
+				} else {
+					entries[i]->highlighted = True;
+				}
 				drawentry(entries[i]);
 			}
 			continue;
 		}
+		if (entries[i]->pressed == True) {
+			entries[i]->pressed = False;
+			drawentry(entries[i]);
+		}
 		if (entries[i]->highlighted == True) {
 			entries[i]->highlighted = False;
 			drawentry(entries[i]);
@@ -132,6 +145,8 @@ buttonpress(XEvent *e)
 	XButtonPressedEvent *ev = &e->xbutton;
 	Entry *en;
 
+	ispressing = True;
+
 	if((en = findentry(ev->x, ev->y)))
 		press(en);
 }
@@ -142,8 +157,10 @@ buttonrelease(XEvent *e)
 	XButtonPressedEvent *ev = &e->xbutton;
 	Entry *en;
 
+	ispressing = False;
+
 	if((en = findentry(ev->x, ev->y)))
-		unpress();
+		unpress(en);
 }
 
 void
@@ -285,8 +302,10 @@ initfont(const char *fontstr)
 		XFreeFontSet(dpy, dc.font.set);
 	dc.font.set = XCreateFontSet(dpy, fontstr, &missing, &n, &def);
 	if(missing) {
-		while(n--)
-			fprintf(stderr, "svkbd: missing fontset: %s\n", missing[n]);
+		while(n--) {
+			fprintf(stderr, "thingmenu: missing fontset: %s\n",
+					missing[n]);
+		}
 		XFreeStringList(missing);
 	}
 	if(dc.font.set) {
@@ -316,38 +335,7 @@ initfont(const char *fontstr)
 void
 leavenotify(XEvent *e)
 {
-	unpress();
-}
-
-void
-runentry(Entry *e)
-{
-	char *shell;
-
-	if (fork()) {
-		if (oneshot || e->forceexit) {
-			XDestroyWindow(dpy, win);
-			exit(0);
-		}
-		return;
-	}
-	if (fork())
-		exit(0);
-
-	shell = getenv("SHELL");
-	if (!shell)
-		shell = "/bin/sh";
-
-	execlp(shell, basename(shell), "-c", e->cmd, (char *)NULL);
-}
-
-void
-press(Entry *e)
-{
-	e->pressed = !e->pressed;
-
-	runentry(e);
-	drawentry(e);
+	unpress(NULL);
 }
 
 void
@@ -384,17 +372,28 @@ setup(void)
 
 	/* init appearance */
 
+	for (i = 0, www = 0; i < nentries; i++) {
+		ls = textnw(entries[i]->label,
+				strlen(entries[i]->label));
+		if (ls > www)
+			www = ls;
+	}
+	www *= widthscaling;
+
 	if (ww == 0) {
-		for (i = 0, ww = 0; i < nentries; i++) {
-			ls = textnw(entries[i]->label,
-					strlen(entries[i]->label));
-			if (ls > ww)
-				ww = ls;
+		if (horizontal) {
+			ww = www * nentries;
+		} else {
+			ww = www;
 		}
-		ww *= 1.5;
+	}
+	if (wh == 0) {
+		if (horizontal) {
+			wh = dc.font.height * heightscaling;
+		} else {
+			wh = nentries * dc.font.height * heightscaling;
+		}
 	}
-	if (wh == 0)
-		wh = (nentries + 2) * dc.font.height + 4;
 	if (wy == 0)
 		wy = (sh - wh) / 2;
 	if (wx == 0)
@@ -458,14 +457,51 @@ textnw(const char *text, uint len)
 }
 
 void
-unpress()
+runentry(Entry *e)
+{
+	char *shell;
+
+	if (fork()) {
+		if (oneshot || e->forceexit) {
+			XDestroyWindow(dpy, win);
+			exit(0);
+		}
+		return;
+	}
+	if (fork())
+		exit(0);
+
+	shell = getenv("SHELL");
+	if (!shell)
+		shell = "/bin/sh";
+
+	execlp(shell, basename(shell), "-c", e->cmd, (char *)NULL);
+}
+
+void
+press(Entry *e)
+{
+	e->pressed = !e->pressed;
+
+	drawentry(e);
+}
+
+void
+unpress(Entry *e)
 {
 	int i;
 
-	for(i = 0; i < nentries; i++) {
-		if(entries[i]->pressed) {
-			entries[i]->pressed = 0;
-			drawentry(entries[i]);
+	if (e != NULL) {
+		e->pressed = !e->pressed;
+
+		runentry(e);
+		drawentry(e);
+	} else {
+		for(i = 0; i < nentries; i++) {
+			if(entries[i]->pressed) {
+				entries[i]->pressed = 0;
+				drawentry(entries[i]);
+			}
 		}
 	}
 }
@@ -473,40 +509,57 @@ unpress()
 void
 updateentries(void)
 {
-	int i, y = 0, h;
+	int i, x, y, h, w;
 
-	h = wh / nentries;
+	x = 0;
+	y = 0;
+
+	if (horizontal) {
+		h = wh;
+		w = www;
+	} else {
+		h = wh / nentries;
+		w = ww;
+	}
 	for(i = 0; i < nentries; i++) {
-		entries[i]->x = 0;
+		entries[i]->x = x;
 		entries[i]->y = y;
-		entries[i]->w = ww;
+		entries[i]->w = w;
 		entries[i]->h = h;
-		y += h;
+		if (horizontal) {
+			x += w;
+		} else {
+			y += h;
+		}
 	}
 }
 
 void
 usage(char *argv0)
 {
-	fprintf(stderr, "usage: %s [-hs] [-wh height] [-ww width] "
-			"[-wx x position] [-wy y position] [--] "
-			"label:cmd ...\n", argv0);
+	fprintf(stderr, "usage: %s [-hxso] [-wh height] [-ww width] "
+			"[-wx x position] [-wy y position] [-ws widthscaling] "
+			"[-hs heightscaling] [--] "
+			"label0 cmd0 [label1 cmd1 ...]\n", argv0);
 	exit(1);
 }
 
 int
 main(int argc, char *argv[])
 {
+	Bool addexit;
 	char *label, *cmd;
 	int i;
 
+
+	addexit = True;
+
 	if (argc < 2)
 		usage(argv[0]);
 	i = 1;
 	for (; argv[i]; i++) {
 		if (argv[i][0] != '-')
 			break;
-
 		if (argv[i][1] == '-') {
 			i++;
 			break;
@@ -514,7 +567,18 @@ main(int argc, char *argv[])
 
 		switch (argv[i][1]) {
 		case 'h':
-			usage(argv[0]);
+			switch ((i >= argc - 1)? 0 : argv[i][2]) {
+			case 's':
+				heightscaling = atof(argv[i+1]);
+				i++;
+				break;
+			default:
+				usage(argv[0]);
+			}
+			break;
+		case 'o':
+			horizontal = True;
+			break;
 		case 's':
 			oneshot = 0;
 			break;
@@ -524,6 +588,10 @@ main(int argc, char *argv[])
 				wh = atoi(argv[i+1]);
 				i++;
 				break;
+			case 's':
+				widthscaling = atof(argv[i+1]);
+				i++;
+				break;
 			case 'w':
 				ww = atoi(argv[i+1]);
 				i++;
@@ -540,6 +608,9 @@ main(int argc, char *argv[])
 				usage(argv[0]);
 			}
 			break;
+		case 'x':
+			addexit = False;
+			break;
 		default:
 			usage(argv[0]);
 		}
@@ -547,16 +618,14 @@ main(int argc, char *argv[])
 
 	for (; argv[i]; i++) {
 		label = argv[i];
-		cmd = strchr(label, ':');
-		if (cmd == NULL) {
-			cmd = label;
-		} else {
-			*cmd++ = '\0';
-		}
+		if (!argv[i+1])
+			break;
+		i++;
+		cmd = argv[i];
 
 		entries = realloc(entries, sizeof(entries[0])*(++nentries));
 		entries[nentries-1] = malloc(sizeof(*entries[0]));
-		bzero(entries[nentries-1], sizeof(*entries[0]));
+		memset(entries[nentries-1], 0, sizeof(*entries[0]));
 
 		entries[nentries-1]->label = strdup(label);
 		if (entries[nentries-1]->label == NULL)
@@ -568,12 +637,14 @@ main(int argc, char *argv[])
 	if (nentries < 1)
 		usage(argv[0]);
 
-	entries = realloc(entries, sizeof(entries[0])*(++nentries));
-	entries[nentries-1] = malloc(sizeof(*entries[0]));
-	bzero(entries[nentries-1], sizeof(*entries[0]));
-	entries[nentries-1]->label = strdup("cancel");
-	entries[nentries-1]->cmd = "exit";
-	entries[nentries-1]->forceexit = True;
+	if (addexit) {
+		entries = realloc(entries, sizeof(entries[0])*(++nentries));
+		entries[nentries-1] = malloc(sizeof(*entries[0]));
+		memset(entries[nentries-1], 0, sizeof(*entries[0]));
+		entries[nentries-1]->label = strdup("cancel");
+		entries[nentries-1]->cmd = "exit";
+		entries[nentries-1]->forceexit = True;
+	}
 
 	if(!setlocale(LC_CTYPE, "") || !XSupportsLocale())
 		fprintf(stderr, "warning: no locale support\n");
